/*    
	imbutton.cpp
	
	QButton representing an IM contact
	
	Copyright (c) 2004 Will Stephenson   <lists@stevello.free-online.co.uk>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <qtooltip.h>
#include <kiconloader.h>
#include <qbuffer.h>

#include "kimproxy.h"

#include "imbutton.h"

IMButton::IMButton( QWidget *parent, KIMProxy *proxy ) : QPushButton( parent )
{
	m_proxy = proxy;
	connect( m_proxy, SIGNAL( sigContactStatusChanged( QString ) ), this, SLOT( statusChanged( ) ) );
	connect( this, SIGNAL( clicked() ), this, SLOT( slotClicked() ) );
	setPixmap( SmallIcon( "metacontact_unknown" ) );
}

IMButton::IMButton( QWidget *parent, KIMProxy *proxy, const QString& name, const QString& contactId )
	: QPushButton( parent )
{
	m_proxy = proxy;
	connect( m_proxy, SIGNAL( sigContactStatusChanged( QString ) ), this, SLOT( statusChanged( ) ) );
	connect( this, SIGNAL( clicked() ), this, SLOT( slotClicked() ) );
	//setPixmap( SmallIcon( "metacontact_unknown" ) );
	setContact( name, contactId );
}

IMButton::~IMButton()
{
}

void IMButton::setContact( const QString& name, const QString& contactId )
{
	m_name = name;
	m_contact = contactId;
	statusChanged();
}

QString IMButton::contact()
{
	return m_contact;
}

void IMButton::statusChanged()
{
	QToolTip::remove( this );
	if ( !m_contact.isNull() )
	{
		// update our tooltip
		QString text = 
			QString("%1 : %2").arg( m_name ).arg( m_proxy->statusString( m_contact ) );
		QToolTip::add( this, text );
		// update our icon
		
		QPixmap pixmap = m_proxy->statusIcon ( m_contact );
		setPixmap( pixmap );
/*		switch ( m_proxy->statusNumeric( m_contact ) )
		{
		case -1:
		case 0:
			setPixmap( SmallIcon( "metacontact_unknown" ) );
			break;
		case 1:
		case 2:
			setPixmap( SmallIcon( "metacontact_offline" ) );break;
		case 3:
			setPixmap( SmallIcon( "metacontact_away" ) );
			break;
		case 4:
			setPixmap( SmallIcon( "metacontact_online" ) );
			break;
		}*/
	}
}

void IMButton::slotClicked()
{
	if ( !m_contact.isNull() )
	{
		m_proxy->chatContactById( m_contact );
	}
}


#include "imbutton.moc"
