/*    
	imapplet.cpp
	
	Proof of concept taskbar applet using KIMProxy IM service library
	
	Copyright (c) 2004 Will Stephenson   <lists@stevello.free-online.co.uk>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <qfont.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qvbox.h>
#include <qsizepolicy.h>
#include <qtimer.h>

#include <dcopclient.h>
#include <kaboutapplication.h>
#include <kaboutdata.h>
#include <kdialogbase.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>
#include <klistview.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kmessagebox.h>
//#include <kpassivepopup.h>
#include <ktoolbar.h>

#include <kabc/addressbook.h>
#include <kabc/stdaddressbook.h>
#include <kabc/addresseedialog.h>

#include "kimproxy.h"
#include "imapplet.h"

#include "imbutton.h"
#include "imlabel.h"

IMApplet::IMApplet(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
    : KPanelApplet(configFile, type, actions, parent, name)
{
    // Get the current application configuration handle
    ksConfig = config();
    
    m_dcopClient = new DCOPClient();
    m_dcopClient->registerAs("IMApplet");
    m_imProxy = new KIMProxy( m_dcopClient );
	
	//
	//box.setAutoAdd(true);
	// Set up the choose contact dialog
	m_dialog = new KDialogBase(this, "choose_contact", true, i18n("Choose Contact"), KDialogBase::Ok|KDialogBase::Cancel );
	m_dialog->enableButton( KDialogBase::Ok, false );
	QVBox *page = m_dialog->makeVBoxMainWidget();
	m_addresseeListView = new KListView( page, "addressees");
	m_addresseeListView->addColumn( "Contacts" );
	//setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum ) ); 

// 	// set up button
// 	m_layout = new QVBoxLayout(this);
// 	m_button = new IMButton( this, m_imProxy );
// 	m_button->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
// 	m_layout->addWidget( m_button );

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	m_label = new IMLabel( this, m_imProxy );

	QVBoxLayout *box = new QVBoxLayout( this );
	box->add( m_label );
	/*QLabel *status  = new QLabel( QString::fromLatin1("Status"), this );
	status->setFont( QFont( "Helvetica", 8 ) );
	box->add( status );*/	
	// Start to load the addressbook and notify us when it's ready
	m_addressBook = 0L;
	m_addressBook = KABC::StdAddressBook::self( true );
	connect( m_addressBook, SIGNAL( loadingFinished( Resource * ) ), this, SLOT( slotLoadAddressees() ) );
	connect( m_addressBook, SIGNAL( loadingFinished( Resource * ) ), this, SLOT( loadPrefs()));
	
	//if ( m_label->addressee().isEmpty() )
	//	preferences();
}

IMApplet::~IMApplet()
{
	delete m_imProxy;
	delete m_dcopClient;
}

void IMApplet::slotLoadAddressees()
{
	m_dialog->enableButton( KDialogBase::Ok, false );
	if ( m_addressBook)
	{
		m_addresseeListView->clear();
		m_imProxy->initialize();
		QStringList addressees = m_imProxy->imAddresseeUids();
		//KMessageBox::information(0, QString("Got %1 addressees from IM App").arg( addressees.count() ) );
		QStringList::Iterator end = addressees.end();
		for ( QStringList::Iterator i = addressees.begin(); i != end; ++i )
		{
			KABC::Addressee candidate = m_addressBook->findByUid( *i );
			int strangers = 0;
			if ( candidate.isEmpty() )
				strangers += 1;
			else
				new KABC::AddresseeItem( m_addresseeListView, candidate );
			if ( strangers )
				KMessageBox::error(0, QString("The IM app believes there are %i IMable contacts\nwhich the addressbook does not recognise.").arg( strangers) );
				
		}
		if ( addressees.count() == 0 ) 
			new KListViewItem( m_addresseeListView, "No IMable contacts found!");
		else
			m_dialog->enableButton( KDialogBase::Ok, true );
	}
	else
		KMessageBox::error(0, QString::fromLatin1("No bloody address book!") );
}

void IMApplet::about()
{
    //KMessageBox::information(0, i18n("IMApplet is a proof-of-concept implementation of a client for the KIMProxy generic instant messenging services over DCOP library."));
	KAboutData aboutData( "IMApplet", I18N_NOOP("IMApplet"), I18N_NOOP("v 0.0"), I18N_NOOP("IMApplet is a proof-of-concept implementation of a client for the\nKIMProxy 'generic instant messaging services over DCOP' library."), KAboutData::License_GPL_V2, "(c) 2004 Will Stephenson", 0 ,0 );
	aboutData.addAuthor("Will Stephenson", I18N_NOOP("Developer"), "lists@stevello.free-online.co.uk", 0 );
	
	KAboutApplication about(&aboutData);
	
	about.setCaption( i18n("About IMApplet") );
	about.exec();
}


void IMApplet::help()
{
    KMessageBox::information(0, i18n("Select a new contact to represent using Configure IMApplet."));
}

void IMApplet::preferences()
{
	// show the choose contact dialog
	if (m_dialog->exec() == QDialog::Accepted )
	{
		// get the UID of the selected contact
		KABC::AddresseeItem *item = 0L;
		item = dynamic_cast<KABC::AddresseeItem *>( m_addresseeListView->selectedItem() );
		if ( item )
		{
	// 		m_button->setAddressee( item->addressee() );
			m_label->setAddressee( item->addressee() );
		}
		//KMessageBox::information(0, QString("w: %1\nh: %2\nmy_w: %3\nmy_h: %4").arg( m_label->size().width() ).arg( m_label->size().height() ).arg( size().width() ).arg(size().height() ) );
		savePrefs();
	}
}

void IMApplet::loadPrefs()
{
    KConfig *kcConfig = config();
    kcConfig->reparseConfiguration();
	kcConfig->setGroup("General Options");
	QString contact = kcConfig->readEntry("contact");
	if ( !contact.isEmpty() && m_addressBook )
	{
		KABC::Addressee candidate = m_addressBook->findByUid( contact );
// 		m_button->setAddressee( candidate );
 		m_label->setAddressee( candidate );
	}
}

void IMApplet::savePrefs()
{
	KConfig *kcConfig = config();

	kcConfig->setGroup("General Options");
	kcConfig->writeEntry("contact", m_label->addressee().uid() );
	kcConfig->sync();
}

int IMApplet::widthForHeight(int height) const
{
    return height;
}

int IMApplet::heightForWidth(int width) const
{
    return width;
}

void IMApplet::resizeEvent(QResizeEvent *)
{
}


extern "C"
{
    KPanelApplet* init( QWidget *parent, const QString configFile)
    {
        KGlobal::locale()->insertCatalogue("IMApplet");
        return new IMApplet(configFile, KPanelApplet::Normal,
                             KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
                             parent, "IMApplet");
    }
}

#include "imapplet.moc"
